package main

import (
	//	"bytes"
	"flag"
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"os/exec"
	"path"
	"strconv"
)

func check(e error) {
	if e != nil {
		//panic(e)
		log.Println("ERROR: ", e)
	}
}

func orientSubject(inputDir string) {
	subjectName := path.Base(inputDir)

	log.Println("Processing ", inputDir)
	//inFileName := inputDir + "/anat/003/"+subjectName+"_mpr003.nii.gz"
	//outFileName := inputDir + "/anat/003/"+subjectName+"_mpr003_reorient.nii.gz"
	inFileName := inputDir + "/anat/003/anat_brain.nii.gz"
	outFileName := inputDir + "/anat/003/anat_brain_reorient.nii.gz"
	log.Println("Renaming", outFileName, "to", inFileName)
	err := os.Rename(outFileName, inFileName)
	check(err)

	log.Println("Executing: fslswapdim", inFileName, "-z -x -y", outFileName)
	cmd := exec.Command("fslswapdim", inFileName, "-z", "-x", "-y", outFileName)
	//var out bytes.Buffer
	//cmd.Stdout = &out
	err = cmd.Run()
	check(err)
	//fmt.Printf(out.String())

	log.Println("Executing: fslorient -forceradiological", outFileName)
	cmd = exec.Command("fslorient", "-forceradiological", outFileName)
	err = cmd.Run()
	check(err)

	var zeroes string
	for boldId := 8; boldId < 14; boldId++ {
		if boldId < 10 {
			zeroes = "00"
		} else {
			zeroes = "0"
		}
		boldIdStr := zeroes + strconv.Itoa(boldId)
		inFileName = inputDir + "/bold/" + boldIdStr + "/" + subjectName + "_bld" + boldIdStr + "_rest.nii.gz"
		outFileName = inputDir + "/bold/" + boldIdStr + "/" + subjectName + "_bld" + boldIdStr + "_rest_reorient.nii.gz"
		log.Println("Renaming", outFileName, "to", inFileName)
		err = os.Rename(outFileName, inFileName)
		check(err)

		log.Println("Executing: fslswapdim", inFileName, "-x -z -y", outFileName)
		cmd = exec.Command("fslswapdim", inFileName, "-x", "-z", "-y", outFileName)
		err = cmd.Run()
		check(err)
		log.Println("Executing: fslorient -forceradiological", outFileName)
		cmd = exec.Command("fslorient", "-forceradiological", outFileName)
		err = cmd.Run()
		check(err)
	}
}

// Convert all subjects in subdirectories using convertSubject()
func orientAll(inputDir string) {
	files, err := ioutil.ReadDir(inputDir)
	check(err)

	for _, f := range files {
		if f.IsDir() {
			inputSubjectDir := inputDir + "/" + f.Name()
			if _, err := os.Stat(inputSubjectDir + "/" + "RunInfo.csv"); err == nil {
				orientSubject(inputSubjectDir)
			} else {
				log.Println("File RunInfo.csv does not exist in " + inputSubjectDir + ". Skipping directory.")
			}
		}
	}
}

func main() {
	// check arguments and flags
	flag.Parse()
	arguments := flag.Args()
	if len(arguments) != 1 {
		fmt.Println("len(arguments = ", len(arguments))
		fmt.Println("Usage: ", path.Base(os.Args[0]), " inputDirectory")
		flag.PrintDefaults()
		os.Exit(0)
	}
	orientAll(path.Clean(arguments[0]))
}
